package com.gic.haoban.manage.service.task;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.ExternalUserDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.MemberUnionidRelatedApiService;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.redis.data.util.RedisUtil;

@Service
public class FriendMergeSyncOperation implements BaseSyncOperation{
	private static final Logger logger = LoggerFactory.getLogger(FriendMergeSyncOperation.class);
	@Autowired
	private MemberUnionidRelatedApiService memberUnionidRelatedApiService;
	@Autowired
	private PreDealService preDealService;
	@Autowired
	private SyncTaskService syncTaskService;
	@Override
	public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
		String reason = "";
		boolean dealFlag = true;
		String taskId = dealParamMqDTO.getTaskId();
		try {
			String dataContent = dataPre.getDataContent();
			ExternalUserDTO pojo = JSON.parseObject(dataContent, ExternalUserDTO.class);
			String res = memberUnionidRelatedApiService.addExternal(pojo);
			if(StringUtils.isBlank(res)){
				dealSuccess(dealParamMqDTO.getTaskId(), dataPre.getDataId(), null, null);
			}else{
				dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), "好友不存在");
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.info("同步失败：{},{}", JSONObject.toJSONString(dataPre),e);
            reason = "合并处理异常";
            dealFlag = false;
		}finally {
			if (!dealFlag) {
               dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), reason);
           }
			checkDepartmentTask(taskId);		
		}
		
	}

	@Override
	public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
		preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.exception.getVal(), reason);
		
	}

	@Override
	public void dealSuccess(String taskId, String dataId, String enterpriseId, String wxEnterpriseId) {
		preDealService.updateStatusByDataId(taskId, dataId, PreDealStatusEnum.computed.getVal(), "成功");
		
	}

	@Override
	public void checkDepartmentTask(String taskId) {
		String key = "sync_wx_lock_";
		boolean b = preDealService.checkTask(taskId, PreDealTypeEnum.friend_merge.getVal());
        if (!b) {
            return;
        }
        logger.info("好友合并成功:{}", taskId);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        int errorCount = preDealService.countByTaskId(taskId, PreDealTypeEnum.friend_merge.getVal(), 3);
        if(errorCount > 0){
        	syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_compute.getVal());
        }else{
        	syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
        }
        //同步成功 进入门店处理
    	RedisUtil.delCache(key + syncTask.getWxEnterpriseId());
	}

}
